@@ -130,40 +130,40 @@ void Main::readInAFile(SATSolver* solver2, const string& filename)
         exit(-1);
     }
 
-    if (!independent_vars_str.empty() && !parser.independent_vars.empty()) {
-        cerr << "ERROR! Independent vars set in console but also in CNF." << endl;
+    if (!sampling_vars_str.empty() && !parser.sampling_vars.empty()) {
+        cerr << "ERROR! Sampling vars set in console but also in CNF." << endl;
         exit(-1);
     }
 
-    if (!independent_vars_str.empty()) {
-        assert(independent_vars.empty());
+    if (!sampling_vars_str.empty()) {
+        assert(sampling_vars.empty());
 
-        std::stringstream ss(independent_vars_str);
+        std::stringstream ss(sampling_vars_str);
         uint32_t i;
         while (ss >> i)
         {
             const uint32_t var = i-1;
-            independent_vars.push_back(var);
+            sampling_vars.push_back(var);
 
             if (ss.peek() == ',' || ss.peek() == ' ')
                 ss.ignore();
         }
     } else {
-        independent_vars.swap(parser.independent_vars);
+        sampling_vars.swap(parser.sampling_vars);
     }
 
-    if (independent_vars.empty()) {
-        if (only_indep_solution) {
-            cout << "ERROR: only independent vars are requested in the solution, but no independent vars have been set!" << endl;
+    if (sampling_vars.empty()) {
+        if (only_sampling_solution) {
+            cout << "ERROR: only sampling vars are requested in the solution, but no sampling vars have been set!" << endl;
             exit(-1);
         }
     } else {
-        solver2->set_independent_vars(&independent_vars);
-        cout << "c Independent vars set: ";
-        for(size_t i = 0; i < independent_vars.size(); i++) {
-            const uint32_t v = independent_vars[i];
+        solver2->set_sampling_vars(&sampling_vars);
+        cout << "c Sampling vars set: ";
+        for(size_t i = 0; i < sampling_vars.size(); i++) {
+            const uint32_t v = sampling_vars[i];
             cout << v+1;
-            if (i+1 != independent_vars.size())
+            if (i+1 != sampling_vars.size())
                 cout << ",";
         }
         cout << endl;
@@ -213,13 +213,14 @@ void Main::readInStandardInput(SATSolver* solver2)
 
 void Main::parseInAllFiles(SATSolver* solver2)
 {
+    const double myTimeTotal = cpuTimeTotal();
     const double myTime = cpuTime();
 
     //First read normal extra files
     if (!debugLib.empty() && filesToRead.size() > 1) {
         cout
-        << "debugLib must be OFF"
-        << "to parse in more than one file"
+        << "ERROR: debugLib must be OFF"
+        << " to parse in more than one file"
         << endl;
 
         std::exit(-1);
@@ -235,11 +236,19 @@ void Main::parseInAllFiles(SATSolver* solver2)
     }
 
     if (conf.verbosity) {
-        cout
-        << "c Parsing time: "
-        << std::fixed << std::setprecision(2)
-        << (cpuTime() - myTime)
-        << " s" << endl;
+        if (num_threads > 1) {
+            cout
+            << "c Sum parsing time among all threads (wall time will differ): "
+            << std::fixed << std::setprecision(2)
+            << (cpuTimeTotal() - myTimeTotal)
+            << " s" << endl;
+        } else {
+            cout
+            << "c Parsing time: "
+            << std::fixed << std::setprecision(2)
+            << (cpuTime() - myTime)
+            << " s" << endl;
+        }
     }
 }
 
@@ -275,8 +284,8 @@ void Main::printResultFunc(
         } else {
             const uint32_t num_undef = print_model(os, solver);
             if (num_undef && !toFile && conf.verbosity) {
-                if (only_indep_solution) {
-                    cout << "c NOTE: some variables' value are NOT set -- you ONLY asked for the independent set's values: '--onlyindep'" << endl;
+                if (only_sampling_solution) {
+                    cout << "c NOTE: some variables' value are NOT set -- you ONLY asked for the sampling set's values: '--onlysampling'" << endl;
                 } else {
                    cout << "c NOTE: " << num_undef << " variables are NOT set" << endl;
                 }
@@ -358,14 +367,16 @@ void Main::add_supported_options()
         , "Glue value for lev 1 cut ('give another shot'")
     ("adjustglue", po::value(&conf.adjust_glue_if_too_many_low)->default_value(conf.adjust_glue_if_too_many_low, s_adjust_low.str())
         , "If more than this % of clauses is LOW glue (level 0) then lower the glue cutoff by 1 -- once and never again")
-    ("ml", po::value(&conf.guess_cl_effectiveness)->default_value(conf.guess_cl_effectiveness)
-        , "Use ML model to guess clause effectiveness")
     ("everylev1", po::value(&conf.every_lev1_reduce)->default_value(conf.every_lev1_reduce)
         , "Reduce lev1 clauses every N")
     ("everylev2", po::value(&conf.every_lev2_reduce)->default_value(conf.every_lev2_reduce)
         , "Reduce lev2 clauses every N")
     ("lev1usewithin", po::value(&conf.must_touch_lev1_within)->default_value(conf.must_touch_lev1_within)
         , "Learnt clause must be used in lev1 within this timeframe or be dropped to lev2")
+    ;
+
+    po::options_description red_cl_dump_opts("Clause dumping after problem finishing");
+    reduceDBOptions.add_options()
     ("dumpred", po::value(&dump_red_fname)->default_value(dump_red_fname)
         , "Dump redundant clauses of gluecut0&1 to this filename")
     ("dumpredmaxlen", po::value(&dump_red_max_len)->default_value(dump_red_max_len)
@@ -405,7 +416,7 @@ void Main::add_supported_options()
     po::options_description iterativeOptions("Iterative solve options");
     iterativeOptions.add_options()
     ("maxsol", po::value(&max_nr_of_solutions)->default_value(max_nr_of_solutions)
-        , "Search for given amount of solutions")
+        , "Search for given amount of solutions. Thanks to Jannis Harder for the decision-based banning idea")
     ("debuglib", po::value<string>(&debugLib)
         , "MainSolver at specific 'solve()' points in CNF file")
     ("dumpresult", po::value(&resultFilename)
@@ -431,8 +442,8 @@ void Main::add_supported_options()
     std::ostringstream ssERatio;
     ssERatio << std::setprecision(4) << "norm: " << conf.varElimRatioPerIter << " preproc: " << 1.0;
 
-    po::options_description simplificationOptions("Simplification options");
-    simplificationOptions.add_options()
+    po::options_description simp_schedules("Simplification schedules");
+    simp_schedules.add_options()
     ("schedsimp", po::value(&conf.do_simplify_problem)->default_value(conf.do_simplify_problem)
         , "Perform simplification rounds. If 0, we never perform any.")
     ("presimp", po::value(&conf.simplify_at_startup)->default_value(conf.simplify_at_startup)
@@ -455,16 +466,38 @@ void Main::add_supported_options()
         , "Start first simplification after this many conflicts")
     ("confbtwsimpinc", po::value(&conf.num_conflicts_of_search_inc)->default_value(conf.num_conflicts_of_search_inc)
         , "Simp rounds increment by this power of N")
-    ("varelim", po::value(&conf.doVarElim)->default_value(conf.doVarElim)
-        , "Perform variable elimination as per Een and Biere")
-    ("varelimto", po::value(&conf.varelim_time_limitM)->default_value(conf.varelim_time_limitM)
-        , "Var elimination bogoprops M time limit")
-    ("varelimover", po::value(&conf.min_bva_gain)->default_value(conf.min_bva_gain)
-        , "Do BVE until the resulting no. of clause increase is less than X. Only power of 2 makes sense, i.e. 2,4,8...")
-    ("emptyelim", po::value(&conf.do_empty_varelim)->default_value(conf.do_empty_varelim)
-        , "Perform empty resolvent elimination using bit-map trick")
+    ;
+
+    po::options_description tern_res_options("Ternary resolution");
+    tern_res_options.add_options()
+    ("tern", po::value(&conf.doTernary)->default_value(conf.doTernary)
+        , "Perform Ternary resolution'")
+    ("terntimelim", po::value(&conf.ternary_res_time_limitM)->default_value(conf.ternary_res_time_limitM)
+        , "Time-out in bogoprops M of ternary resolution as per paper 'Look-Ahead Versus Look-Back for Satisfiability Problems'")
+    ;
+
+    po::options_description occ_mem_limits("Occ-based simplification memory limits");
+    occ_mem_limits.add_options()
+    ("occredmax", po::value(&conf.maxRedLinkInSize)->default_value(conf.maxRedLinkInSize)
+        , "Don't add to occur list any redundant clause larger than this")
+    ("occredmaxmb", po::value(&conf.maxOccurRedMB)->default_value(conf.maxOccurRedMB)
+        , "Don't allow redundant occur size to be beyond this many MB")
+    ("occirredmaxmb", po::value(&conf.maxOccurIrredMB)->default_value(conf.maxOccurIrredMB)
+        , "Don't allow irredundant occur size to be beyond this many MB")
+    ;
+
+    po::options_description sub_str_time_limits("Occ-based subsumption and strengthening time limits");
+    sub_str_time_limits.add_options()
     ("strengthen", po::value(&conf.do_strengthen_with_occur)->default_value(conf.do_strengthen_with_occur)
         , "Perform clause contraction through self-subsuming resolution as part of the occurrence-subsumption system")
+    ("substimelim", po::value(&conf.subsumption_time_limitM)->default_value(conf.subsumption_time_limitM)
+        , "Time-out in bogoprops M of subsumption of long clauses with long clauses, after computing occur")
+    ("strstimelim", po::value(&conf.strengthening_time_limitM)->default_value(conf.strengthening_time_limitM)
+        , "Time-out in bogoprops M of strengthening of long clauses with long clauses, after computing occur")
+    ;
+
+    po::options_description bva_options("BVA options");
+    bva_options.add_options()
     ("bva", po::value(&conf.do_bva)->default_value(conf.do_bva)
         , "Perform bounded variable addition")
     ("bvalim", po::value(&conf.bva_limit_per_call)->default_value(conf.bva_limit_per_call)
@@ -473,20 +506,24 @@ void Main::add_supported_options()
         , "BVA with 2-lit difference hack, too. Beware, this reduces the effectiveness of 1-lit diff")
     ("bvato", po::value(&conf.bva_time_limitM)->default_value(conf.bva_time_limitM)
         , "BVA time limit in bogoprops M")
+    ;
+
+    po::options_description bve_options("BVE options");
+    bve_options.add_options()
+    ("varelim", po::value(&conf.doVarElim)->default_value(conf.doVarElim)
+        , "Perform variable elimination as per Een and Biere")
+    ("varelimto", po::value(&conf.varelim_time_limitM)->default_value(conf.varelim_time_limitM)
+        , "Var elimination bogoprops M time limit")
+    ("varelimover", po::value(&conf.min_bva_gain)->default_value(conf.min_bva_gain)
+        , "Do BVE until the resulting no. of clause increase is less than X. Only power of 2 makes sense, i.e. 2,4,8...")
+    ("emptyelim", po::value(&conf.do_empty_varelim)->default_value(conf.do_empty_varelim)
+        , "Perform empty resolvent elimination using bit-map trick")
+    ("varelimmaxmb", po::value(&conf.var_linkin_limit_MB)->default_value(conf.var_linkin_limit_MB)
+        , "Maximum extra MB of memory to use for new clauses during varelim")
     ("eratio", po::value(&conf.varElimRatioPerIter)->default_value(conf.varElimRatioPerIter, ssERatio.str())
         , "Eliminate this ratio of free variables at most per variable elimination iteration")
     ("skipresol", po::value(&conf.skip_some_bve_resolvents)->default_value(conf.skip_some_bve_resolvents)
         , "Skip BVE resolvents in case they belong to a gate")
-    ("occredmax", po::value(&conf.maxRedLinkInSize)->default_value(conf.maxRedLinkInSize)
-        , "Don't add to occur list any redundant clause larger than this")
-    ("occirredmaxmb", po::value(&conf.maxOccurIrredMB)->default_value(conf.maxOccurIrredMB)
-        , "Don't allow irredundant occur size to be beyond this many MB")
-    ("occredmaxmb", po::value(&conf.maxOccurRedMB)->default_value(conf.maxOccurRedMB)
-        , "Don't allow redundant occur size to be beyond this many MB")
-    ("substimelim", po::value(&conf.subsumption_time_limitM)->default_value(conf.subsumption_time_limitM)
-        , "Time-out in bogoprops M of subsumption of long clauses with long clauses, after computing occur")
-    ("strstimelim", po::value(&conf.strengthening_time_limitM)->default_value(conf.strengthening_time_limitM)
-        , "Time-out in bogoprops M of strengthening of long clauses with long clauses, after computing occur")
     ("agrelimtimelim", po::value(&conf.aggressive_elim_time_limitM)->default_value(conf.aggressive_elim_time_limitM)
         , "Time-out in bogoprops M of aggressive(=uses reverse distillation) var-elimination")
     ;
@@ -554,6 +591,12 @@ void Main::add_supported_options()
         , "Always strong-minimise clause")
     ("otfsubsume", po::value(&conf.doOTFSubsume)->default_value(conf.doOTFSubsume)
         , "Perform on-the-fly subsumption")
+    ("decbased", po::value(&conf.do_decision_based_cl)->default_value(conf.do_decision_based_cl)
+        , "Create decision-based conflict clauses when the UIP clause is too large")
+    ("decbasemaxlev", po::value(&conf.decision_based_cl_max_levels)->default_value(conf.decision_based_cl_max_levels)
+        , "Create decision-based conflict if the maximum level is below or equal to this")
+    ("decbaseminsz", po::value(&conf.decision_based_cl_min_learned_size)->default_value(conf.decision_based_cl_min_learned_size)
+        , "Create decision-based conflict if the learnt clause is larger than this")
     ;
 
     po::options_description propOptions("Propagation options");
@@ -632,6 +675,9 @@ void Main::add_supported_options()
         , "Renumber variables to increase CPU cache efficiency")
     ("savemem", po::value(&conf.doSaveMem)->default_value(conf.doSaveMem)
         , "Save memory by deallocating variable space after renumbering. Only works if renumbering is active.")
+    ("fullwatchconseveryn", po::value(&conf.full_watch_consolidate_every_n_confl)->default_value(conf.full_watch_consolidate_every_n_confl)
+        , "Consolidate watchlists fully once every N conflicts. Scheduled during simplification rounds.")
+
     ("implicitmanip", po::value(&conf.doStrSubImplicit)->default_value(conf.doStrSubImplicit)
         , "Subsume and strengthen implicit clauses with each other")
     ("implsubsto", po::value(&conf.subsume_implicit_time_limitM)->default_value(conf.subsume_implicit_time_limitM)
@@ -657,21 +703,25 @@ void Main::add_supported_options()
         , "Interrupt threads cleanly, all the time")
     ("zero-exit-status", po::bool_switch(&zero_exit_status)
         , "Exit with status zero in case the solving has finished without an issue")
-    ("input", po::value< vector<string> >(), "file(s) to read")
     ("printtimes", po::value(&conf.do_print_times)->default_value(conf.do_print_times)
         , "Print time it took for each simplification run. If set to 0, logs are easier to compare")
-    ("drat,d", po::value(&dratfilname)
-        , "Put DRAT verification information into this file")
     ("savedstate", po::value(&conf.saved_state_file)->default_value(conf.saved_state_file)
         , "The file to save the saved state of the solver")
     ("maxsccdepth", po::value(&conf.max_scc_depth)->default_value(conf.max_scc_depth)
         , "The maximum for scc search depth")
     ("simdrat", po::value(&conf.simulate_drat)->default_value(conf.simulate_drat)
-        , "The maximum for scc search depth")
-    ("indep", po::value(&independent_vars_str)->default_value(independent_vars_str)
-        , "Independent vars, separated by comma")
-    ("onlyindep", po::bool_switch(&only_indep_solution)
-        , "Independent vars, separated by comma")
+        , "Simulate DRAT")
+    ("dumpdecformodel", po::value(&decisions_for_model_fname)->default_value(decisions_for_model_fname)
+        , "Decisions for model will be dumped here")
+    ("sampling", po::value(&sampling_vars_str)->default_value(sampling_vars_str)
+        , "Sampling vars, separated by comma")
+    ("onlysampling", po::bool_switch(&only_sampling_solution)
+        , "Print and ban(!) solutions only in terms of variables declared in 'c ind' or as --sampling '...'")
+
+    //these a kind of special and determine positional options' meanings
+    ("input", po::value< vector<string> >(), "file(s) to read")
+    ("drat,d", po::value(&dratfilname)
+        , "Put DRAT verification information into this file")
     ;
 
 #ifdef USE_GAUSS
@@ -701,15 +751,21 @@ void Main::add_supported_options()
     .add(printOptions)
     .add(propOptions)
     .add(reduceDBOptions)
+    .add(red_cl_dump_opts)
     .add(varPickOptions)
     .add(conflOptions)
     .add(iterativeOptions)
     .add(probeOptions)
     .add(stampOptions)
-    .add(simplificationOptions)
+    .add(simp_schedules)
+    .add(occ_mem_limits)
+    .add(tern_res_options)
+    .add(sub_str_time_limits)
+    .add(bve_options)
+    .add(bva_options)
     .add(eqLitOpts)
     .add(componentOptions)
-    #if defined(USE_M4RI) or defined(USE_GAUSS)
+    #if defined(USE_M4RI) || defined(USE_GAUSS)
     .add(xorOptions)
     #endif
     .add(gateOptions)
@@ -955,7 +1011,7 @@ void Main::parse_polarity_type()
 void Main::manually_parse_some_options()
 {
     if (conf.maxXorToFind > MAX_XOR_RECOVER_SIZE) {
-        cout << "ERROR: The '--maxxorsize' parameter cannot be lager than " << MAX_XOR_RECOVER_SIZE << endl;
+        cout << "ERROR: The '--maxxorsize' parameter cannot be larger than " << MAX_XOR_RECOVER_SIZE << endl;
         exit(-1);
     }
 
@@ -973,6 +1029,19 @@ void Main::manually_parse_some_options()
         exit(-1);
     }
 
+    if (!decisions_for_model_fname.empty() && max_nr_of_solutions > 1) {
+        std::cerr << "ERROR: dumping decisions for multi-solution makes no sense. Exiting." << endl;
+        std::exit(-1);
+    }
+
+    if (!decisions_for_model_fname.empty()) {
+        conf.need_decisions_reaching = true;
+    }
+
+    if (max_nr_of_solutions > 1) {
+        conf.need_decisions_reaching = true;
+    }
+
     if (conf.preprocess != 0) {
         conf.simplify_at_startup = 1;
         conf.varelim_time_limitM *= 5;
@@ -1002,6 +1071,11 @@ void Main::manually_parse_some_options()
             std::exit(-1);
         }
 
+        if (!decisions_for_model_fname.empty()) {
+            std::cerr << "ERROR: dumping decisions for model make no sense with preprocessing. Exiting." << endl;
+            std::exit(-1);
+        }
+
         if (!filesToRead.empty()) {
             assert(false && "we should never reach this place, filesToRead has not been populated yet");
             exit(-1);
@@ -1038,7 +1112,7 @@ void Main::manually_parse_some_options()
             cout
             << "ERROR: Couldn't open file '"
             << resultFilename
-            << "' for writing!"
+            << "' for writing result!"
             << endl;
             std::exit(-1);
         }
@@ -1075,6 +1149,10 @@ void Main::manually_parse_some_options()
         conf.solution_file = solution[0];
     } else if (vm.count("input")) {
         filesToRead = vm["input"].as<vector<string> >();
+        if (conf.preprocess == 1) {
+            filesToRead.resize(1);
+        }
+
         if (!vm.count("sqlitedb")) {
             sqlite_filename = filesToRead[0] + ".sqlite";
         } else {
@@ -1086,11 +1164,25 @@ void Main::manually_parse_some_options()
     }
 
     if (conf.preprocess == 1) {
-        if (!vm.count("drat")) {
+        if (vm["input"].as<vector<string> >().size() + vm.count("drat") > 2) {
             cout << "ERROR: When preprocessing, you must give the simplified file name as 2nd argument" << endl;
-            std::exit(-1);
+            cout << "You gave this many inputs: "
+                << vm["input"].as<vector<string> >().size()+vm.count("drat")
+                << endl;
+
+            for(string s: vm["input"].as<vector<string> >()) {
+                cout << " --> " << s << endl;
+            }
+            if (vm.count("drat")) {
+                cout << " --> " << vm["drat"].as<string>() << endl;
+            }
+            exit(-1);
+        }
+        if (vm["input"].as<vector<string> >().size() > 1) {
+            conf.simplified_cnf = vm["input"].as<vector<string> >()[1];
+        } else {
+            conf.simplified_cnf = vm["drat"].as<string>();
         }
-        conf.simplified_cnf = vm["drat"].as<string>();
     }
 
     if (conf.preprocess == 2) {
@@ -1175,7 +1267,7 @@ void Main::dump_red_file()
         cout
         << "ERROR: Couldn't open file '"
         << resultFilename
-        << "' for writing!"
+        << "' for writing redundant clauses!"
         << endl;
         std::exit(-1);
     }
@@ -1257,13 +1349,48 @@ int Main::solve()
     return correctReturnValue(ret);
 }
 
+void Main::dump_decisions_for_model()
+{
+    assert(max_nr_of_solutions == 1);
+    assert(solver->okay());
+
+    std::ofstream decfile;
+    decfile.open(decisions_for_model_fname.c_str());
+    if (!(decfile)) {
+        cout
+        << "ERROR: Couldn't open file '"
+        << decisions_for_model_fname
+        << "' for writing decisions!"
+        << endl;
+        std::exit(-1);
+    }
+
+    if (!solver->get_decision_reaching_valid()) {
+        decfile << "INVALID" << endl;
+        return;
+    }
+
+    if (conf.verbosity) {
+        cout << "c size of get_decisions_reaching_model: "
+        << solver->get_decisions_reaching_model().size()
+        << endl;;
+    }
+    for(const Lit l: solver->get_decisions_reaching_model()) {
+        decfile << l << " 0" << endl;
+    }
+}
+
 lbool Main::multi_solutions()
 {
     unsigned long current_nr_of_solutions = 0;
     lbool ret = l_True;
     while(current_nr_of_solutions < max_nr_of_solutions && ret == l_True) {
-        ret = solver->solve(NULL, only_indep_solution);
+        ret = solver->solve(NULL, only_sampling_solution);
         current_nr_of_solutions++;
+        if (ret == l_True && !decisions_for_model_fname.empty()) {
+            dump_decisions_for_model();
+            assert(max_nr_of_solutions == 1);
+        }
 
         if (ret == l_True && current_nr_of_solutions < max_nr_of_solutions) {
             printResultFunc(&cout, false, ret);
@@ -1283,14 +1410,22 @@ lbool Main::multi_solutions()
 
             //Banning found solution
             vector<Lit> lits;
-            if (independent_vars.empty()) {
-              for (uint32_t var = 0; var < solver->nVars(); var++) {
-                  if (solver->get_model()[var] != l_Undef) {
-                      lits.push_back( Lit(var, (solver->get_model()[var] == l_True)? true : false) );
-                  }
-              }
+            if (sampling_vars.empty()) {
+                if (solver->get_decision_reaching_valid()) {
+                    //only decision vars
+                    for (Lit lit: solver->get_decisions_reaching_model()) {
+                      lits.push_back(~lit);
+                    }
+                } else {
+                    //all of the solution
+                    for (uint32_t var = 0; var < solver->nVars(); var++) {
+                        if (solver->get_model()[var] != l_Undef) {
+                            lits.push_back( Lit(var, (solver->get_model()[var] == l_True)? true : false) );
+                        }
+                    }
+                }
             } else {
-              for (const uint32_t var: independent_vars) {
+              for (const uint32_t var: sampling_vars) {
                   if (solver->get_model()[var] != l_Undef) {
                       lits.push_back( Lit(var, (solver->get_model()[var] == l_True)? true : false) );
                   }
