@@ -69,6 +69,7 @@ class Main
         void handle_drat_option();
         void parse_restart_type();
         void parse_polarity_type();
+        void dump_decisions_for_model();
         void check_num_threads_sanity(const unsigned thread_num) const;
 
         po::positional_options_description p;
@@ -113,9 +114,13 @@ class Main
         uint32_t max_nr_of_solutions = 1;
         int sql = 0;
         string sqlite_filename;
-        vector<uint32_t> independent_vars;
-        std::string independent_vars_str = "";
-        bool only_indep_solution = false;
+        string decisions_for_model_fname;
+
+        //Sampling vars
+        vector<uint32_t> sampling_vars;
+        std::string sampling_vars_str = "";
+        bool only_sampling_solution = false;
+
 
         //Files to read & write
         bool fileNamePresent;
