@@ -122,16 +122,16 @@ OccSimplifier::~OccSimplifier()
 void OccSimplifier::new_var(const uint32_t /*orig_outer*/)
 {
     n_occurs.insert(n_occurs.end(), 2, 0);
-    if (solver->conf.independent_vars) {
-        indep_vars.insert(indep_vars.end(), 1, 0);
+    if (solver->conf.sampling_vars) {
+        sampling_vars_occsimp.insert(sampling_vars_occsimp.end(), 1, 0);
     }
 }
 
 void OccSimplifier::new_vars(size_t n)
 {
     n_occurs.insert(n_occurs.end(), n*2ULL, 0);
-    if (solver->conf.independent_vars) {
-        indep_vars.insert(indep_vars.end(), n, 0);
+    if (solver->conf.sampling_vars) {
+        sampling_vars_occsimp.insert(sampling_vars_occsimp.end(), n, 0);
     }
 }
 
@@ -236,6 +236,7 @@ void OccSimplifier::extend_model(SolutionExtender* extender)
         Lit blockedOn = solver->varReplacer->get_lit_replaced_with_outer(it->at(0, blkcls));
         size_t at = 1;
         bool satisfied = false;
+        lits.clear();
         while(at < it->size()) {
             if (it->at(at, blkcls) == lit_Undef) {
                 if (!satisfied) {
@@ -244,8 +245,9 @@ void OccSimplifier::extend_model(SolutionExtender* extender)
                     #ifndef DEBUG_VARELIM
                     //all should be satisfied in fact
                     //no need to go any further
-                    if (var_set)
+                    if (var_set) {
                         break;
+                    }
                     #endif
                 }
                 satisfied = false;
@@ -253,12 +255,12 @@ void OccSimplifier::extend_model(SolutionExtender* extender)
             } else if (!satisfied) {
                 Lit l = it->at(at, blkcls);
                 l = solver->varReplacer->get_lit_replaced_with_outer(l);
+                lits.push_back(l);
 
                 //Blocked clause can be skipped, it's satisfied
                 if (solver->model_value(l) == l_True) {
                     satisfied = true;
                 }
-                lits.push_back(l);
             }
             at++;
         }
@@ -570,36 +572,6 @@ OccSimplifier::LinkInData OccSimplifier::link_in_clauses(
     return link_in_data;
 }
 
-bool OccSimplifier::decide_occur_limit(bool irred, uint64_t memUsage)
-{
-    //over + irred -> exit
-    if (irred
-        && (double)memUsage/(1024.0*1024.0) >= solver->conf.maxOccurIrredMB
-    ) {
-        if (solver->conf.verbosity) {
-            cout
-            << "c [simp] Not linking in irred due to excessive expected memory usage"
-            << endl;
-        }
-        return false;
-    }
-
-    //over + red -> don't link
-    if (!irred
-        && (double)memUsage/(1024.0*1024.0) >= solver->conf.maxOccurRedMB
-    ) {
-        if (solver->conf.verbosity) {
-            cout
-            << "c [simp] Not linking in red due to excessive expected memory usage"
-            << endl;
-        }
-
-        return false;
-    }
-
-    return true;
-}
-
 bool OccSimplifier::check_varelim_when_adding_back_cl(const Clause* cl) const
 {
     bool notLinkedNeedFree = false;
@@ -759,9 +731,9 @@ void OccSimplifier::eliminate_empty_resolvent_vars()
 bool OccSimplifier::can_eliminate_var(const uint32_t var) const
 {
     #ifdef SLOW_DEBUG
-    if (solver->conf.independent_vars) {
+    if (solver->conf.sampling_vars) {
         assert(var < solver->nVars());
-        assert(var < indep_vars.size());
+        assert(var < sampling_vars_occsimp.size());
     }
     #endif
 
@@ -769,7 +741,7 @@ bool OccSimplifier::can_eliminate_var(const uint32_t var) const
     if (solver->value(var) != l_Undef
         || solver->varData[var].removed != Removed::none
         || solver->var_inside_assumptions(var)
-        || (solver->conf.independent_vars && indep_vars[var])
+        || (solver->conf.sampling_vars && sampling_vars_occsimp[var])
         //|| (!solver->conf.allow_elim_xor_vars && solver->varData[var].added_for_xor)
     ) {
         return false;
@@ -1013,6 +985,7 @@ bool OccSimplifier::eliminate_vars()
     last_trail = solver->trail_size();
 
     while(varelim_num_limit > 0
+        && varelim_linkin_limit_bytes > 0
         && *limit_to_decrease > 0
         && grow < (uint32_t)solver->conf.min_bva_gain
         //&& grow < 1 //solver->conf.min_bva_gain
@@ -1040,6 +1013,7 @@ bool OccSimplifier::eliminate_vars()
             while(!velim_order.empty()
                 && *limit_to_decrease > 0
                 && varelim_num_limit > 0
+                && varelim_linkin_limit_bytes > 0
                 && !solver->must_interrupt_asap()
             ) {
                 assert(limit_to_decrease == &norm_varelim_time_limit);
@@ -1152,6 +1126,17 @@ bool OccSimplifier::eliminate_vars()
             cout << "c Reduced to " << solver->get_num_free_vars() << " vars"
             << ", " << sum_irred_cls_longs() + solver->binTri.irredBins
             << " cls (grow=" << grow << ")" << endl;
+
+            if (varelim_num_limit < 0
+                || varelim_linkin_limit_bytes < 0
+                || *limit_to_decrease < 0
+            ) {
+                cout << "c [occ-bve] stopped varelim due to outage. "
+                << " varelim_num_limit: "; print_value_kilo_mega(varelim_num_limit);
+                cout << " varelim_linkin_limit_bytes: "; print_value_kilo_mega(varelim_linkin_limit_bytes);
+                cout << " *limit_to_decrease: "; print_value_kilo_mega(*limit_to_decrease);
+                cout << endl;
+            }
         }
 
 
@@ -1193,9 +1178,9 @@ end:
     }
     if (solver->conf.verbosity) {
         if (solver->conf.verbosity >= 3)
-            runStats.print(solver->nVarsOuter());
+            runStats.print(solver->nVarsOuter(), this);
         else
-            runStats.print_short();
+            runStats.print_extra_times();
     }
     if (solver->sqlStats) {
         solver->sqlStats->time_passed(
@@ -1321,7 +1306,6 @@ bool OccSimplifier::execute_simplifier_strategy(const string& strategy)
         }
 
         #ifdef SLOW_DEBUG
-        check_clid_correct();
         solver->check_implicit_stats(true);
         #endif
         if (!solver->propagate_occur()) {
@@ -1337,6 +1321,10 @@ bool OccSimplifier::execute_simplifier_strategy(const string& strategy)
         }
         if (token == "occ-backw-sub-str") {
             backward_sub_str();
+        } else if (token == "occ-ternary-res") {
+            if (solver->conf.doTernary) {
+                ternary_res();
+            }
         } else if (token == "occ-xor") {
             if (solver->conf.doFindXors) {
                 XorFinder finder(this, solver);
@@ -1363,6 +1351,7 @@ bool OccSimplifier::execute_simplifier_strategy(const string& strategy)
                     n_occurs[lit.toInt()] = calc_occ_data(lit);
                     n_occurs[(~lit).toInt()] = calc_occ_data(~lit);
                 }
+                runStats.xorTime += finder.get_stats().findTime;
             }
         } else if (token == "occ-clean-implicit") {
             //BUG TODO
@@ -1470,19 +1459,19 @@ bool OccSimplifier::simplify(const bool _startup, const std::string schedule)
     const size_t origTrailSize = solver->trail_size();
 
 
-    indep_vars.clear();
-    if (solver->conf.independent_vars) {
-        indep_vars.resize(solver->nVars(), false);
-        for(uint32_t outside_var: *solver->conf.independent_vars) {
+    sampling_vars_occsimp.clear();
+    if (solver->conf.sampling_vars) {
+        sampling_vars_occsimp.resize(solver->nVars(), false);
+        for(uint32_t outside_var: *solver->conf.sampling_vars) {
             uint32_t outer_var = solver->map_to_with_bva(outside_var);
             outer_var = solver->varReplacer->get_var_replaced_with_outer(outer_var);
             uint32_t int_var = solver->map_outer_to_inter(outer_var);
             if (int_var < solver->nVars()) {
-                indep_vars[int_var] = true;
+                sampling_vars_occsimp[int_var] = true;
             }
         }
     } else {
-        indep_vars.shrink_to_fit();
+        sampling_vars_occsimp.shrink_to_fit();
     }
 
     execute_simplifier_strategy(schedule);
@@ -1493,6 +1482,173 @@ bool OccSimplifier::simplify(const bool _startup, const std::string schedule)
     return solver->okay();
 }
 
+bool OccSimplifier::ternary_res()
+{
+    assert(solver->okay());
+    assert(cl_to_add_ternary.empty());
+    if (clauses.empty()) {
+        return solver->okay();
+    }
+
+    double myTime = cpuTime();
+    int64_t orig_ternary_res_time_limit = ternary_res_time_limit;
+    limit_to_decrease = &ternary_res_time_limit;
+
+    //NOTE: the "clauses" here will change in size as we add resolvents
+    size_t at = solver->mtrand.randInt(clauses.size()-1);
+    for(size_t i = 0; i < clauses.size(); i++) {
+        ClOffset offs = clauses[(at+i) % clauses.size()];
+        Clause * cl = solver->cl_alloc.ptr(offs);
+        *limit_to_decrease -= 10;
+        if (!cl->freed()
+            && !cl->getRemoved()
+            && !cl->is_ternary_resolved
+            && cl->size() == 3
+            && !cl->red()
+            && *limit_to_decrease > 0
+        ) {
+            cl->is_ternary_resolved = true;
+            if (!perform_ternary(cl, offs))
+                break;
+        }
+    }
+
+    //Update global stats
+    const double time_used = cpuTime() - myTime;
+    const bool time_out = (*limit_to_decrease <= 0);
+    const double time_remain =  float_div(*limit_to_decrease, orig_ternary_res_time_limit);
+    if (solver->conf.verbosity) {
+        cout
+        << "c [occ-ternary-res] Ternary res added: " << runStats.ternary_added
+        << solver->conf.print_times(time_used, time_out, time_remain)
+        << endl;
+    }
+    if (solver->sqlStats) {
+        solver->sqlStats->time_passed(
+            solver
+            , "ternary res"
+            , time_used
+            , time_out
+            , time_remain
+        );
+    }
+    runStats.triresolveTime += time_used;
+
+    return solver->okay();
+}
+
+bool OccSimplifier::perform_ternary(Clause* cl, ClOffset offs)
+{
+    assert(cl_to_add_ternary.empty());
+    *limit_to_decrease -= 3;
+    for(const Lit l: *cl) {
+        seen[l.toInt()] = 1;
+    }
+
+    size_t largest = 0;
+    Lit dont_check = lit_Undef;
+    for(const Lit l: *cl) {
+        size_t sz = n_occurs[l.toInt()] + n_occurs[(~l).toInt()];
+        if (sz > largest) {
+            largest = sz;
+            dont_check = l;
+        }
+    }
+
+    for(const Lit l: *cl) {
+        if (l == dont_check) {
+            continue;
+        }
+        check_ternary_cl(cl, offs, solver->watches[l]);
+        check_ternary_cl(cl, offs, solver->watches[~l]);
+    }
+
+    //clean up
+    for(const Lit l: *cl) {
+        seen[l.toInt()] = 0;
+    }
+
+    //Add new ternary resolvents
+    for(vector<Lit>& newcl: cl_to_add_ternary) {
+        Clause* newCl = solver->add_clause_int(
+            newcl //Literals in new clause
+            , true //Is the new clause redundant?
+            , ClauseStats() //Statistics for this new clause (usage, etc.)
+            , false //Should clause be attached if long?
+        );
+        *limit_to_decrease-=20;
+
+        if (!solver->ok)
+            break;
+
+        if (newCl != NULL) {
+            newCl->stats.glue = 3;
+            linkInClause(*newCl);
+            ClOffset offset = solver->cl_alloc.get_offset(newCl);
+            clauses.push_back(offset);
+        }
+    }
+    cl_to_add_ternary.clear();
+
+    return solver->okay();
+}
+
+void OccSimplifier::check_ternary_cl(Clause* cl, ClOffset offs, watch_subarray ws)
+{
+    *limit_to_decrease -= ws.size()*2;
+    for (Watched& w: ws) {
+        if (!w.isClause() || w.get_offset() == offs)
+            continue;
+
+        ClOffset offs2 = w.get_offset();
+        Clause * cl2 = solver->cl_alloc.ptr(offs2);
+        *limit_to_decrease -= 10;
+        if (!cl2->freed()
+            && !cl2->getRemoved()
+            && cl2->size() == 3
+            && !cl2->red()
+        ) {
+            uint32_t num_lits = 3;
+            uint32_t num_vars = 3;
+            Lit lit_clash = lit_Undef;
+            for(Lit l2: *cl2) {
+                num_vars += !(seen[l2.toInt()] | seen[(~l2).toInt()]);
+                num_lits += !seen[l2.toInt()];
+                if (seen[(~l2).toInt()]) {
+                    lit_clash = l2;
+
+                    //It's symmetric so only do it one way
+                    if (!lit_clash.sign()) {
+                        lit_clash = lit_Error;
+                        break;
+                    }
+                }
+            }
+
+            //Not tri resolveeable or the wrong side of the symmetry
+            if (num_vars != 4 || num_lits != 5 || lit_clash == lit_Error) {
+                continue;
+            }
+            vector<Lit> newcl;
+            for(Lit l: *cl) {
+                if (l.var() != lit_clash.var())
+                    newcl.push_back(l);
+            }
+            for(Lit l: *cl2) {
+                if (l.var() != lit_clash.var()
+                    && !seen[l.toInt()]
+                ) {
+                    newcl.push_back(l);
+                }
+            }
+            cl_to_add_ternary.push_back(newcl);
+            *limit_to_decrease-=20;
+            runStats.ternary_added++;
+            //cout << "tri: " << *cl << " , " << *cl2 << " Resolve on: " << lit_clash << endl;
+        }
+    }
+}
+
 bool OccSimplifier::backward_sub_str()
 {
     limit_to_decrease = &subsumption_time_limit;
@@ -1830,6 +1986,8 @@ void OccSimplifier::set_limits()
         *solver->conf.global_timeout_multiplier;
     varelim_sub_str_limit      = 1000ULL*1000ULL*solver->conf.varelim_sub_str_limit
         *solver->conf.global_timeout_multiplier;
+    ternary_res_time_limit     = 1000ULL*1000ULL*solver->conf.ternary_res_time_limitM
+        *solver->conf.global_timeout_multiplier;
 
     //If variable elimination isn't going so well
     if (bvestats_global.testedToElimVars > 0
@@ -1849,6 +2007,7 @@ void OccSimplifier::set_limits()
     varelim_sub_str_limit *= 10;
 
     varelim_num_limit = ((double)solver->get_num_free_vars() * solver->conf.varElimRatioPerIter);
+    varelim_linkin_limit_bytes = solver->conf.var_linkin_limit_MB *1000LL*1000LL*solver->conf.var_and_mem_out_mult;
 
     if (!solver->conf.do_strengthen_with_occur) {
         strengthening_time_limit = 0;
@@ -1868,13 +2027,6 @@ void OccSimplifier::set_limits()
 //     varelim_sub_str_limit    = std::numeric_limits<int64_t>::max();
 }
 
-void OccSimplifier::cleanBlockedClausesIfDirty()
-{
-    if (can_remove_blocked_clauses) {
-        cleanBlockedClauses();
-    }
-}
-
 void OccSimplifier::cleanBlockedClauses()
 {
     assert(solver->decisionLevel() == 0);
@@ -2330,13 +2482,19 @@ bool OccSimplifier::add_varelim_resolvent(
         ClOffset offset = solver->cl_alloc.get_offset(newCl);
         clauses.push_back(offset);
         added_long_cl.push_back(offset);
+
+        // 4 = clause itself
+        // 8 = watch space
+        varelim_linkin_limit_bytes -= finalLits.size()*(4+8);
+        varelim_linkin_limit_bytes -= sizeof(Clause);
+
     } else if (finalLits.size() == 2) {
         added_bin_cl.push_back(std::make_pair(finalLits[0], finalLits[1]));
         n_occurs[finalLits[0].toInt()]++;
         n_occurs[finalLits[1].toInt()]++;
-        if (!solver->ok) {
-            return false;
-        }
+
+        // 8 = watch space
+        varelim_linkin_limit_bytes -= finalLits.size()*(8);
     }
 
     //Touch every var of the new clause, so we re-estimate
@@ -2901,7 +3059,7 @@ size_t OccSimplifier::mem_used() const
     b += varElimComplexity.capacity()*sizeof(int)*2;
     b += elim_calc_need_update.mem_used();
     b += clauses.capacity()*sizeof(ClOffset);
-    b += indep_vars.capacity();
+    b += sampling_vars_occsimp.capacity();
 
     return b;
 }
@@ -2922,13 +3080,6 @@ size_t OccSimplifier::mem_used_bva() const
         return 0;
 }
 
-
-void OccSimplifier::freeXorMem()
-{
-    delete topLevelGauss;
-    topLevelGauss = NULL;
-}
-
 void OccSimplifier::linkInClause(Clause& cl)
 {
     assert(cl.size() > 2);
@@ -2958,10 +3109,15 @@ void OccSimplifier::linkInClause(Clause& cl)
     }
 }*/
 
-double OccSimplifier::Stats::total_time() const
+double OccSimplifier::Stats::total_time(OccSimplifier* occs) const
 {
-    return linkInTime + blockTime
-        + varElimTime + finalCleanupTime;
+    return linkInTime + varElimTime + xorTime + triresolveTime
+        + finalCleanupTime
+        + occs->sub_str->get_stats().subsumeTime
+        + occs->sub_str->get_stats().strengthenTime
+        + occs->bvestats_global.timeUsed
+        + occs->bva->get_stats().time_used;
+        ;
 }
 
 void OccSimplifier::Stats::clear()
@@ -2976,10 +3132,12 @@ OccSimplifier::Stats& OccSimplifier::Stats::operator+=(const Stats& other)
 
     //Time
     linkInTime += other.linkInTime;
-    blockTime += other.blockTime;
     varElimTime += other.varElimTime;
+    xorTime += other.xorTime;
+    triresolveTime += other.triresolveTime;
     finalCleanupTime += other.finalCleanupTime;
     zeroDepthAssings += other.zeroDepthAssings;
+    ternary_added += other.ternary_added;
 
     return *this;
 }
@@ -2989,7 +3147,6 @@ BVEStats& BVEStats::operator+=(const BVEStats& other)
     numVarsElimed += other.numVarsElimed;
     varElimTimeOut += other.varElimTimeOut;
     clauses_elimed_long += other.clauses_elimed_long;
-    clauses_elimed_tri += other.clauses_elimed_tri;
     clauses_elimed_bin += other.clauses_elimed_bin;
     clauses_elimed_sumsize += other.clauses_elimed_sumsize;
     longRedClRemThroughElim += other.longRedClRemThroughElim;
@@ -3003,7 +3160,7 @@ BVEStats& BVEStats::operator+=(const BVEStats& other)
     return *this;
 }
 
-void OccSimplifier::Stats::print_short() const
+void OccSimplifier::Stats::print_extra_times() const
 {
 
     cout
@@ -3016,24 +3173,21 @@ void OccSimplifier::Stats::print_short() const
     << endl;
 }
 
-void OccSimplifier::Stats::print(const size_t nVars) const
+void OccSimplifier::Stats::print(const size_t nVars, OccSimplifier* occs) const
 {
     cout << "c -------- OccSimplifier STATS ----------" << endl;
     print_stats_line("c time"
-        , total_time()
-        , stats_line_percent(varElimTime, total_time())
+        , total_time(occs)
+        , stats_line_percent(varElimTime, total_time(occs))
         , "% var-elim"
     );
 
     print_stats_line("c called"
         ,  numCalls
-        , float_div(total_time(), numCalls)
+        , float_div(total_time(occs), numCalls)
         , "s per call"
     );
 
-
-
-
     print_stats_line("c 0-depth assigns"
         , zeroDepthAssings
         , stats_line_percent(zeroDepthAssings, nVars)
@@ -3079,15 +3233,3 @@ void OccSimplifier::load_state(SimpleInFile& f)
         }
     }
 }
-
-void OccSimplifier::check_clid_correct() const
-{
-    #ifdef STATS_NEEDED
-    for(auto offs: clauses) {
-        Clause * cl = solver->cl_alloc.ptr(offs);
-        if (!cl->freed() && !cl->getRemoved()) {
-            assert(!(cl->stats.ID == 0 && cl->red()));
-        }
-    }
-    #endif
-}
