@@ -75,7 +75,6 @@ class Searcher : public HyperEngine
         //
         lbool solve(
             uint64_t max_confls
-            , const unsigned upper_level_iteration_num
         );
         void finish_up_solve(lbool status);
         void reduce_db_if_needed();
@@ -88,7 +87,8 @@ class Searcher : public HyperEngine
 
 
         vector<lbool>  model;
-        vector<lbool>  full_model;
+        vector<Lit>    decisions_reaching_model; // the decisions needed to reach current model
+        bool           decisions_reaching_model_valid = false;
         vector<Lit>   conflict;     ///<If problem is unsatisfiable (possibly under assumptions), this vector represent the final conflict clause expressed in the assumptions.
         template<bool update_bogoprops>
         PropBy propagate();
@@ -122,7 +122,7 @@ class Searcher : public HyperEngine
         bool check_order_heap_sanity() const;
 
         SQLStats* sqlStats = NULL;
-        void consolidate_watches();
+        void consolidate_watches(const bool full);
 
         //Gauss
         #ifdef USE_GAUSS
