@@ -97,9 +97,11 @@ class Solver : public Searcher
         lbool full_model_value (const Lit p) const;  ///<Found model value for lit
         lbool full_model_value (const uint32_t p) const;  ///<Found model value for var
         const vector<lbool>& get_model() const;
+        const vector<Lit>& get_decisions_reaching_model() const;
         const vector<Lit>& get_final_conflict() const;
         vector<pair<Lit, Lit> > get_all_binary_xors() const;
         vector<Xor> get_recovered_xors(bool elongate);
+        bool get_decision_reaching_valid() const;
 
         //get learnt clauses
         void start_getting_small_clauses(uint32_t max_len, uint32_t max_glue);
@@ -348,6 +350,7 @@ class Solver : public Searcher
         //Renumberer
         double calc_renumber_saving();
         void free_unused_watches();
+        uint64_t last_full_watch_consolidate = 0;
         void save_on_var_memory(uint32_t newNumVars);
         void unSaveVarMem();
         size_t calculate_interToOuter_and_outerToInter(
@@ -575,6 +578,11 @@ inline const vector<lbool>& Solver::get_model() const
     return model;
 }
 
+inline const vector<Lit>& Solver::get_decisions_reaching_model() const
+{
+    return decisions_reaching_model;
+}
+
 inline const vector<Lit>& Solver::get_final_conflict() const
 {
     return conflict;
@@ -607,21 +615,16 @@ inline lbool Solver::model_value (const uint32_t p) const
     return model[p];
 }
 
-inline lbool Solver::full_model_value (const Lit p) const
-{
-    return full_model[p.var()] ^ p.sign();
-}
-
-inline lbool Solver::full_model_value  (const uint32_t p) const
-{
-    return full_model[p];
-}
-
 inline void Solver::testing_set_solver_not_fresh()
 {
     fresh_solver = false;
 }
 
+inline bool Solver::get_decision_reaching_valid() const
+{
+    return decisions_reaching_model_valid;
+}
+
 } //end namespace
 
 #endif //SOLVER_H
